#include <dos.h>
#include <stdlib.h>
#include <stdio.h>
#include <malloc.h> 
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <bios.h>

#include "textgrph.h"
#include "main.h"


// Returns the number of clock ticks that have elapsed
unsigned long GetTimerTicks(void)
{
  union _REGS regs;
  unsigned long tc;
  
  regs.h.ah = 0;
  _int86(0x1A, &regs, &regs);
  tc = ((unsigned long)regs.x.cx) << 16;
  tc += regs.x.dx;
  return tc;
}


// Waits for a number of seconds
void WaitSecond(double seconds)
{
  unsigned long count0, count;
  
  count0 = GetTimerTicks();
  count = (unsigned long)(count0 + seconds * 18.2);
  while(GetTimerTicks() < count);
}


// Gets and returns the key pressed
short GetKey(short wait)
{
  short k;
  
  if(wait == 1)
  {
    k = _bios_keybrd(_NKEYBRD_READ);
  } else
  {
    k = _bios_keybrd(_NKEYBRD_READY);
  }
    
  return k;
}


void ClearKeyBuffer(void)
{
  while(_bios_keybrd(_NKEYBRD_READY) != 0) 
  {
    _bios_keybrd(_NKEYBRD_READ);
  } 
}

/* This function places the cursor at the specified
   x, y coordinates */
   
void Cursor(short x, short y)
{
  union _REGS regs;
  
  regs.h.ah = 0x02;
  regs.h.bh = 0;
  regs.h.dh = (char)y;
  regs.h.dl = (char)x;
  
  _int86(0x10, &regs, &regs);
}


/* Sets the cursors size */

void SetCursorSize(short start, short end)
{
  union _REGS regs;
  
  regs.h.ah = 0x01;
  regs.h.ch = (char)start;
  regs.h.cl = (char)end;
  
  _int86(0x10, &regs, &regs);
}


/* Gets information on current cursor position */

void GetCursor(short *x, short *y, short *page, short *start, short *end)
{
  union _REGS inregs, outregs;
  
  inregs.h.ah = 0x03;
  inregs.h.bh = 0;
  
  _int86(0x10, &inregs, &outregs);
  
  *y = (short)outregs.h.dh;
  *x = (short)outregs.h.dl;
  *page = (short)outregs.h.bh;
  *start = (short)outregs.h.ch;
  *end = (short)outregs.h.cl;
}


/* Gets the current video mode */

void GetVideo(short *columns, short *displayMode, short *displayPage)
{
  union _REGS inregs, outregs;
  
  inregs.h.ah = 0x0F;
  
  _int86(0x10, &inregs, &outregs);
  
  *columns = (short)outregs.h.ah;
  *displayMode = (short)outregs.h.al;
  *displayPage = (short)outregs.h.bh;
}


/* Sets the video mode */

void SetVideo(short displayMode)
{
  union _REGS regs;
  
  regs.h.ah = 0x00;
  regs.h.al = (char)displayMode;
  
  _int86(0x10, &regs, &regs);
}


/* Sets the border color */

void SetBorderColor(short color)
{
  union _REGS regs;
  
  regs.h.ah = 0x0B;
  regs.h.bh = 0;
  regs.h.bl = (char)color;
  
  _int86(0x10, &regs, &regs);
}


/* Writes a character to screen */

void WriteChar(char ch, short fcolor, short bcolor)
{
  union _REGS regs;
  
  regs.h.ah = 0x09;
  regs.h.al = ch;
  regs.h.bh = 0;
  regs.h.bl = (char)(bcolor << 4) | (char)fcolor;
  regs.x.cx = 1;
  
  _int86(0x10, &regs, &regs);
}


/* Repeats a character starting at cursor location */

void RepeatChar(char ch, short howMany, short fcolor, short bcolor)
{
  union _REGS regs;
  
  regs.h.ah = 0x09;
  regs.h.al = ch;
  regs.h.bh = 0;
  regs.h.bl = (char)(bcolor << 4) | (char)fcolor;
  regs.x.cx = howMany;
  
  _int86(0x10, &regs, &regs);
}


/* Writes a string to screen */

void WriteString(short fcolor, short bcolor, short x, short y, char *string, ...)
{
  short len;
  short i;
  char temp[120];
  va_list args;

  va_start(args, string);
  vsprintf(temp, string, args);
  va_end(args);
  
  len = strlen(temp);
  
  for (i = 0; i < len; i++)
  {
    Cursor(x + i, y);
    WriteChar(temp[i], fcolor, bcolor);
  }
}

/* Draws a box on screen */

void Box(short x0, short y0, short x1, short y1, short type,
         short fill, short fcolor, short bcolor)
{
  short row;  
  static unsigned char DBL_BOX[6] = {201, 205, 187, 186, 200, 188};
  static unsigned char SGL_BOX[6] = {218, 196, 191, 179, 192, 217};
  static unsigned char BLK_BOX[6] = {0, 0, 0, 0, 0, 0};
  static unsigned char *USE_BOX;
  
  switch (type)
  {
    case DOUBLE_BOX:  USE_BOX = DBL_BOX;
                      break;
    case SINGLE_BOX:  USE_BOX = SGL_BOX;
                      break;
    case BLANK_BOX:   USE_BOX = BLK_BOX;
                      break;
    default:          USE_BOX = DBL_BOX;
                      break;
  }
  
  Cursor(x0, y0);
  WriteChar(USE_BOX[0], fcolor, bcolor);
  Cursor(x1, y0);
  WriteChar(USE_BOX[2], fcolor, bcolor);
  
  Cursor(x0 + 1, y0);
  RepeatChar(USE_BOX[1], x1 - x0 - 1, fcolor, bcolor);
  
  Cursor(x0 + 1, y1);
  RepeatChar(USE_BOX[1], x1 - x0 - 1, fcolor, bcolor);
  
  for (row = y0 + 1; row < y1; row++)
  {
    Cursor(x0, row);
    WriteChar(USE_BOX[3], fcolor, bcolor);
    Cursor(x1, row);
    WriteChar(USE_BOX[3], fcolor, bcolor);
  }
  
  Cursor(x0, y1);
  WriteChar(USE_BOX[4], fcolor, bcolor);
  Cursor(x1, y1);
  WriteChar(USE_BOX[5], fcolor, bcolor);
  
  if (fill == 1)
  {
    for (row = y0 + 1; row < y1; row++)
    {
      Cursor(x0 + 1, row);
      RepeatChar(' ', ((x1 - x0) - 1), fcolor, bcolor);
    }
  }
}


/* Scrolls a part of the screen */

void Scroll(short x, short y, short width, short height, short nbr, short direction)
{
  union _REGS regs;
  
  regs.h.ah = (char)direction;
  regs.h.al = (char)nbr;
  regs.h.ch = (char)y;
  regs.h.cl = (char)x;
  regs.h.dh = (char)y + (char)height;
  regs.h.dl = (char)x + (char)width;
  regs.h.bh = 0;
  
  _int86(0x10, &regs, &regs);
}


/* Hides the cursor */

void CursorOff(void)
{
  union _REGS regs;
  
  regs.h.ah = 0x01;
  regs.x.cx = 0x0F00;
  
  _int86(0x10, &regs, &regs);
}


/* Displays the Cursor */

void CursorOn(void)
{
  union _REGS regs;
  
  regs.h.ah = 0x01;
  regs.x.cx = 0x0607;
  
  _int86(0x10, &regs, &regs);
}


/* Clears the screen in the colors fcolor and bcolor */

void ClearScreen(short fcolor, short bcolor)
{
  union _REGS regs;
  
  regs.h.ah = SCROLL_UP;
  regs.h.al = 0;
  regs.h.ch = 0;
  regs.h.cl = 0;
  regs.h.dh = 24;
  regs.h.dl = 79;
  regs.h.bh = (char)(bcolor << 4) | (char)fcolor;
  
  _int86(0x10, &regs, &regs);
}


/* Displays a grid background */

void Grid(short x0, short y0, short x1, short y1, short fcolor, short bcolor, short type)
{
  short ch, row;
  
  switch (type)
  {
    case 1:  ch = 176;
             break;
    case 2:  ch = 177;
             break;
    case 3:  ch = 178;
             break;
  }
  
  for (row = y0; row < (y1 + 1); row++)
  {
    Cursor(x0, row);
    RepeatChar((char)ch, x1 - x0 + 1, fcolor, bcolor);
  }
}


/* Saves a portion of the screen */

char *SaveScreenArea(short x0, short y0, short x1, short y1)
{
  char *screenBuffer;
  union _REGS regs, outregs;
  short totalSpace;
  short row = y0;
  short col;
  short ctr;
  short trow, tcol, page, curSt, curEnd;
  
  GetCursor(&tcol, &trow, &page, &curSt, &curEnd);
  
  totalSpace = (((y1 - y0 + 1) * (x1 - x0 + 1)) * 2) + 5;
  
  screenBuffer = (char *)malloc(totalSpace);
  if (!screenBuffer)
  {
    printf("\nUnable to allocate %d bytes.\n");
    exit(1);
  }
  
  *(screenBuffer + 0) = (char)x0;
  *(screenBuffer + 1) = (char)y0;
  *(screenBuffer + 2) = (char)x1;
  *(screenBuffer + 3) = (char)y1;
  *(screenBuffer + 4) = (char)page;
  
  ctr = 5;
  
  while (row <= y1)
  {
    col = x0;
    while (col <= x1)
    {
      Cursor(col, row);
      
      regs.h.ah = 0x08;
      regs.h.bh = (char)page;
      
      _int86(0x10, &regs, &outregs);
      
      *(screenBuffer + ctr++) = (char)outregs.h.al;
      *(screenBuffer + ctr++) = (char)outregs.h.ah;
      
      col++;
    }
    row++;
  }
  
  return screenBuffer;
}


/* Restores a portion of the screen */

void RestoreScreenArea(char *buffer)
{
  union _REGS regs;
  short x0, x1, y0, y1, page;
  short ctr = 5;
  short row;
  short col;
  
  x0 = (short)*(buffer + 0);
  y0 = (short)*(buffer + 1);
  x1 = (short)*(buffer + 2);
  y1 = (short)*(buffer + 3);
  page = (short)*(buffer + 4);
  
  row = y0;
  
  while (row <= y1)
  {
    col = x0;
    while (col <= x1)
    {
      Cursor(col, row);
      
      regs.h.ah = 0x09;
      regs.h.bh = (char)page;
      regs.h.al = *(buffer + ctr++);
      regs.h.bl = *(buffer + ctr++);
      regs.x.cx = 1;
      
      _int86(0x10, &regs, &regs);
      
      col++;
    }
    row ++;
  }
  
  free(buffer);
}

